@@ -97,7 +97,7 @@ use super::{LocalExecutor, LocalExecutorPoolBuilder};
 ///
 /// handles.join_all();
 /// ```
-#[derive(Debug)]
+#[derive(Clone, Debug, PartialEq)]
 pub enum Placement {
     /// For the `Unbound` variant, the [`LocalExecutor`]s created by a
     /// [`LocalExecutorPoolBuilder`] are not bound to any CPU.  This is the
@@ -177,7 +177,7 @@ impl Default for Placement {
 /// Please see the documentation for [`Placement`] variants to
 /// understand how `CpuSet` restrictions apply to each variant.  CPUs are
 /// identified via their [`CpuLocation`].
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq)]
 pub struct CpuSet(Vec<CpuLocation>);
 
 impl CpuSet {
@@ -506,6 +506,52 @@ mod test {
         assert_eq!(0, v.len());
     }
 
+    #[test]
+    fn placement_unbound_clone() {
+        assert!(matches!(Placement::Unbound.clone(), Placement::Unbound));
+    }
+
+    #[test]
+    fn placement_fenced_clone() {
+        let set = CpuSet::online().unwrap();
+        let placement = Placement::Fenced(set);
+        let placement_clone = placement.clone();
+        assert_eq!(placement_clone, placement);
+    }
+
+    #[test]
+    fn placement_max_spread_clone() {
+        let set = CpuSet::online().unwrap();
+        let some_placement = Placement::MaxSpread(Some(set));
+        let some_placement_clone = some_placement.clone();
+        assert_eq!(some_placement_clone, some_placement);
+        let none_placement = Placement::MaxSpread(None);
+        let none_placement_clone = none_placement.clone();
+        assert_eq!(none_placement_clone, none_placement);
+    }
+
+    #[test]
+    fn placement_max_pack_clone() {
+        let set = CpuSet::online().unwrap();
+        let some_placement = Placement::MaxPack(Some(set));
+        let some_placement_clone = some_placement.clone();
+        assert_eq!(some_placement_clone, some_placement);
+        let none_placement = Placement::MaxPack(None);
+        let none_placement_clone = none_placement.clone();
+        assert_eq!(none_placement_clone, none_placement);
+    }
+
+    #[test]
+    fn placement_custom_clone() {
+        let set1 = CpuSet::online().unwrap();
+        let set2 = CpuSet::online().unwrap();
+        assert!(set1.is_empty());
+        assert!(set2.is_empty());
+        let vec_set = vec![set1, set2];
+        let placement = Placement::Custom(vec_set);
+        assert_eq!(placement.clone(), placement);
+    }
+
     #[test]
     fn max_spreader_this_machine() {
         let n = 4096;
