@@ -24,30 +24,9 @@
 //! those tasks on the same thread, no thread context switch is necessary when
 //! going between task execution and I/O.
 
-use crate::iou::sqe::SockAddrStorage;
-use ahash::AHashMap;
-use nix::sys::socket::{MsgFlags, SockAddr};
-use std::{
-    cell::RefCell,
-    collections::{BTreeMap, VecDeque},
-    ffi::CString,
-    fmt,
-    io,
-    mem,
-    os::unix::{ffi::OsStrExt, io::RawFd},
-    panic::{RefUnwindSafe, UnwindSafe},
-    path::Path,
-    rc::Rc,
-    sync::{
-        atomic::{AtomicU32, Ordering},
-        Arc,
-    },
-    task::Waker,
-    time::{Duration, Instant},
-};
-
 use crate::{
     io::{FileScheduler, IoScheduler, ScheduledSource},
+    iou::sqe::SockAddrStorage,
     sys,
     sys::{
         DirectIo,
@@ -65,6 +44,27 @@ use crate::{
     Local,
     TaskQueueHandle,
 };
+use ahash::AHashMap;
+use nix::sys::socket::{MsgFlags, SockAddr};
+use smallvec::SmallVec;
+use std::{
+    cell::RefCell,
+    collections::BTreeMap,
+    ffi::CString,
+    fmt,
+    io,
+    mem,
+    os::unix::{ffi::OsStrExt, io::RawFd},
+    panic::{RefUnwindSafe, UnwindSafe},
+    path::Path,
+    rc::Rc,
+    sync::{
+        atomic::{AtomicU32, Ordering},
+        Arc,
+    },
+    task::Waker,
+    time::{Duration, Instant},
+};
 
 /// Waits for a notification.
 pub(crate) struct Parker {
@@ -204,10 +204,11 @@ impl Timers {
     }
 }
 
+type SharedChannelWakerChecker = (SmallVec<[Waker; 1]>, Option<Box<dyn Fn() -> usize>>);
+
 struct SharedChannels {
     id: u64,
-    check_map: BTreeMap<u64, Box<dyn Fn() -> usize>>,
-    wakers_map: BTreeMap<u64, VecDeque<Waker>>,
+    wakers_map: BTreeMap<u64, SharedChannelWakerChecker>,
     connection_wakers: Vec<Waker>,
 }
 
@@ -216,7 +217,6 @@ impl SharedChannels {
         SharedChannels {
             id: 0,
             connection_wakers: Vec::new(),
-            check_map: BTreeMap::new(),
             wakers_map: BTreeMap::new(),
         }
     }
@@ -227,17 +227,15 @@ impl SharedChannels {
             wake!(waker);
         }
 
-        let current_wakers = mem::take(&mut self.wakers_map);
-        for (id, mut pending) in current_wakers.into_iter() {
-            let room = self.check_map.get(&id).unwrap()();
-            let room = std::cmp::min(room, pending.len());
-            for waker in pending.drain(0..room) {
+        for (_, (pending, check)) in self.wakers_map.iter_mut() {
+            if pending.is_empty() {
+                continue;
+            }
+            let room = std::cmp::min(check.as_ref().unwrap()(), pending.len());
+            for waker in pending.drain(0..room).rev() {
                 woke += 1;
                 wake!(waker);
             }
-            if !pending.is_empty() {
-                self.wakers_map.insert(id, pending);
-            }
         }
         woke
     }
@@ -337,7 +335,9 @@ impl Reactor {
         let mut channels = self.shared_channels.borrow_mut();
         let id = channels.id;
         channels.id += 1;
-        let ret = channels.check_map.insert(id, test_function);
+        let ret = channels
+            .wakers_map
+            .insert(id, (Default::default(), Some(test_function)));
         assert!(ret.is_none());
         id
     }
@@ -345,7 +345,6 @@ impl Reactor {
     pub(crate) fn unregister_shared_channel(&self, id: u64) {
         let mut channels = self.shared_channels.borrow_mut();
         channels.wakers_map.remove(&id);
-        channels.check_map.remove(&id);
     }
 
     pub(crate) fn add_shared_channel_connection_waker(&self, waker: Waker) {
@@ -355,8 +354,12 @@ impl Reactor {
 
     pub(crate) fn add_shared_channel_waker(&self, id: u64, waker: Waker) {
         let mut channels = self.shared_channels.borrow_mut();
-        let map = channels.wakers_map.entry(id).or_insert_with(VecDeque::new);
-        map.push_back(waker);
+        let map = channels
+            .wakers_map
+            .entry(id)
+            .or_insert_with(|| (SmallVec::new(), None));
+
+        map.0.push(waker);
     }
 
     pub(crate) fn alloc_dma_buffer(&self, size: usize) -> DmaBuffer {
@@ -635,6 +638,38 @@ impl Reactor {
         source
     }
 
+    pub(crate) fn truncate(&self, raw: RawFd, size: u64) -> Source {
+        let source = self.new_source(raw, SourceType::Truncate, None);
+        self.sys.truncate(&source, size);
+        source
+    }
+
+    pub(crate) fn rename<P, Q>(&self, old_path: P, new_path: Q) -> Source
+    where
+        P: AsRef<Path>,
+        Q: AsRef<Path>,
+    {
+        let source = self.new_source(
+            -1,
+            SourceType::Rename(old_path.as_ref().to_owned(), new_path.as_ref().to_owned()),
+            None,
+        );
+        self.sys.rename(&source);
+        source
+    }
+
+    pub(crate) fn remove_file<P: AsRef<Path>>(&self, path: P) -> Source {
+        let source = self.new_source(-1, SourceType::Remove(path.as_ref().to_owned()), None);
+        self.sys.remove_file(&source);
+        source
+    }
+
+    pub(crate) fn create_dir<P: AsRef<Path>>(&self, path: P, mode: libc::c_int) -> Source {
+        let source = self.new_source(-1, SourceType::CreateDir(path.as_ref().to_owned()), None);
+        self.sys.create_dir(&source, mode);
+        source
+    }
+
     pub(crate) fn close(&self, raw: RawFd) -> Source {
         let source = self.new_source(
             raw,
@@ -759,6 +794,7 @@ impl Reactor {
         self.sys.rush_dispatch(None, &mut woke)?;
         woke += self.process_timers().1;
         woke += self.process_shared_channels();
+        woke += self.sys.flush_syscall_thread();
 
         Ok(woke > 0)
     }
