@@ -3,22 +3,17 @@
 //
 // This product includes software developed at Datadog (https://www.datadoghq.com/). Copyright 2020 Datadog, Inc.
 //
-use crate::uring_sys;
+use crate::{to_io_error, uring_sys};
 use ahash::AHashMap;
 use lockfree::channel::mpsc;
 use log::debug;
 use std::{
-    ffi::CString,
     fmt,
     io,
     io::Error,
     mem::{ManuallyDrop, MaybeUninit},
     net::{Shutdown, TcpStream},
-    os::unix::{
-        ffi::OsStrExt,
-        io::{AsRawFd, FromRawFd, RawFd},
-    },
-    path::Path,
+    os::unix::io::{AsRawFd, FromRawFd, RawFd},
     sync::{
         atomic::{AtomicUsize, Ordering},
         Arc,
@@ -29,6 +24,7 @@ use std::{
     time::Duration,
 };
 
+pub(super) mod blocking;
 pub(crate) mod hardware_topology;
 
 macro_rules! syscall {
@@ -75,20 +71,6 @@ pub(crate) fn fs_hint_extentsize(fd: RawFd, size: usize) -> nix::Result<i32> {
     unsafe { set_fsxattr(fd, &attr) }
 }
 
-pub(crate) fn remove_file(path: &Path) -> io::Result<()> {
-    let path = cstr(path)?;
-    syscall!(unlink(path.as_ptr()))?;
-    Ok(())
-}
-
-pub(crate) fn rename_file(old_path: &Path, new_path: &Path) -> io::Result<()> {
-    let old = cstr(old_path)?;
-    let new = cstr(new_path)?;
-
-    syscall!(rename(old.as_ptr(), new.as_ptr()))?;
-    Ok(())
-}
-
 pub(crate) fn truncate_file(fd: RawFd, size: u64) -> io::Result<()> {
     syscall!(ftruncate(fd, size as i64))?;
     Ok(())
@@ -98,11 +80,6 @@ pub(crate) fn duplicate_file(fd: RawFd) -> io::Result<RawFd> {
     syscall!(dup(fd))
 }
 
-pub(crate) fn sync_open(path: &Path, flags: libc::c_int, mode: libc::c_int) -> io::Result<RawFd> {
-    let path = cstr(path)?;
-    syscall!(open(path.as_ptr(), flags, mode))
-}
-
 pub(crate) fn create_eventfd() -> io::Result<RawFd> {
     syscall!(eventfd(0, libc::O_CLOEXEC))
 }
@@ -113,6 +90,12 @@ pub(crate) fn write_eventfd(eventfd: RawFd) {
     assert_eq!(ret, 8);
 }
 
+pub(crate) fn read_eventfd(eventfd: RawFd) {
+    let mut buf = [1u64; 1];
+    let ret = syscall!(read(eventfd, buf.as_mut_ptr() as _, 8)).unwrap();
+    assert_eq!(ret, 8);
+}
+
 pub(crate) fn send_syscall(fd: RawFd, buf: *const u8, len: usize, flags: i32) -> io::Result<usize> {
     syscall!(send(fd, buf as _, len, flags)).map(|x| x as usize)
 }
@@ -148,13 +131,7 @@ pub(crate) unsafe fn ssptr_to_sockaddr(
     } else {
         nix::sys::socket::sockaddr_storage_to_addr(&storage, len)
     }
-    .map_err(|e| {
-        let err_no = e.as_errno();
-        match err_no {
-            Some(err_no) => io::Error::from_raw_os_error(err_no as _),
-            None => io::Error::new(io::ErrorKind::Other, "Unknown error"),
-        }
-    })
+    .map_err(|e| to_io_error!(e))
 }
 
 pub(crate) fn recvmsg_syscall(
@@ -214,10 +191,6 @@ pub(crate) fn sendmsg_syscall(
     syscall!(sendmsg(fd, &hdr, flags)).map(|x| x as usize)
 }
 
-fn cstr(path: &Path) -> io::Result<CString> {
-    Ok(CString::new(path.as_os_str().as_bytes())?)
-}
-
 mod dma_buffer;
 pub(crate) mod source;
 pub(crate) mod sysfs;
@@ -304,7 +277,7 @@ impl fmt::Debug for OsError {
 pub(crate) struct SleepNotifier {
     id: usize,
     eventfd: std::fs::File,
-    memory: Arc<AtomicUsize>,
+    memory: AtomicUsize,
     foreign_wakes: Mutex<mpsc::Receiver<Waker>>,
     waker_sender: mpsc::Sender<Waker>,
 }
@@ -337,7 +310,7 @@ impl SleepNotifier {
         Ok(Arc::new(Self {
             eventfd,
             id,
-            memory: Arc::new(AtomicUsize::new(0)),
+            memory: AtomicUsize::new(0),
             waker_sender,
             foreign_wakes: Mutex::new(foreign_wakes),
         }))
