@@ -30,6 +30,7 @@ use std::{
     fmt,
     mem::MaybeUninit,
     os::unix::io::RawFd,
+    path::PathBuf,
     rc::Rc,
     task::{Poll, Waker},
     time::Duration,
@@ -56,12 +57,16 @@ pub(crate) enum SourceType {
     Open(CString),
     FdataSync,
     Fallocate,
+    Truncate,
     Close,
     LinkRings,
     Statx(CString, Box<RefCell<libc::statx>>),
     Timeout(TimeSpec64),
     Connect(SockAddr),
     Accept(SockAddrStorage),
+    Rename(PathBuf, PathBuf),
+    CreateDir(PathBuf),
+    Remove(PathBuf),
     Invalid,
     #[cfg(feature = "bench")]
     Noop,
